local _G = _G

-- addon name and namespace
local ADDON, NS = ...

-- local functions
local string_format = string.format
local tinsert       = table.insert
local pairs         = pairs
local tostring      = tostring

local GetPlayerMapPosition = GetPlayerMapPosition
local SetMapToCurrentZone  = SetMapToCurrentZone
local GetMapInfo           = GetMapInfo

-- coloring tools
local Crayon = LibStub:GetLibrary("LibCrayon-3.0")

-- colors
NS.HexColors = {
	Blueish  = "04adcb",
	Brownish = "eda55f",
	GrayOut  = "888888",
	Green    = "00ff00",
	Magenta  = "ff00ff",
	Orange   = "e54100",
	Red      = "ff0000",
	White    = "ffffff",
	Yellow   = "ffff00",
}

-- utilities
local cmdLinePattern = "^ *([^%s]+) *"

function NS:GetArgs(str)
   local ret = {}
   local pos = 0
   
   while true do
     local word
     _, pos, word = string.find(str, cmdLinePattern, pos+1)
	 
     if not word then
       break
     end
	 
     --word = string.lower(word)
     table.insert(ret, word)
   end
   
   return ret
end

function NS:Colorize(color, text)
    if not text then
	    return ""
	end

	if type(text) ~= "string" then
		text = tostring(text)
	end
	
	if NS.HexColors[color] then
		text = text:gsub("(.*)(|r)", "%1%2|cff" .. NS.HexColors[color])
		text = "|cff" .. NS.HexColors[color] .. tostring(text) .. "|r"
	end
	
	return text
end

function NS:ColorizeByValue(value, from, to, ...)
	if value and Crayon then
		from = from or 0
		to   = to or 1
		
		local hexColor = Crayon:GetThresholdHexColor(value, from, to)
		
		local args = {...}
		local ret = {}
		
		for k, v in ipairs(args) do
			v = "|cff" .. hexColor .. v .. "|r"
			
			ret[k] = v
		end
		
		return unpack(ret)
	else
		return ...
	end
end

function NS:ClearTable(tab)
	if type(tab) == "table" then
		for k in pairs(tab) do
			tab[k] = nil
		end
	end
end

function NS:FormatTime(stamp)
	stamp = type(stamp) == "number" and stamp or 0
	
	local days    = floor(stamp/86400)
	local hours   = floor((stamp - days * 86400) / 3600)
	local minutes = floor((stamp - days * 86400 - hours * 3600) / 60)
	local seconds = floor((stamp - days * 86400 - hours * 3600 - minutes * 60))

	return string_format("%02d:%02d:%02d", hours, minutes, seconds)
end

function NS:GetPlayerPosition()
	local x, y = GetPlayerMapPosition("player")
	
	-- no valid position on the current map
	if x <= 0 and y <= 0 then
		-- we can only change map if map is not being displayed by player
		if not _G.WorldMapFrame:IsVisible() then
			-- NOTE: we do not recover original map to reduce load
			SetMapToCurrentZone()
			
			x, y = GetPlayerMapPosition("player")
		end		
	end

	-- microdungeon check
	local mapName, textureWidth, textureHeight, isMicroDungeon, microDungeonName = GetMapInfo()

	if isMicroDungeon then
		if not _G.WorldMapFrame:IsVisible() then
			-- zoom out to the main map of this zone
			ZoomOut()
			
			x, y = GetPlayerMapPosition("player")
		end
	end
	
	return x, y
end
